Package: clordr
Type: Package
Title: Composite Likelihood Inference and Diagnostics for Replicated
        Spatial Ordinal Data
Version: 1.7.2
Authors@R: c(person("Ting Fung", "Ma", email = "tingfung@mailbox.sc.edu", role = c("cre", "aut")),
    person("Pingping", "Wang", email = "catherinewpp@126.com", role = "aut"),
    person("Jun", "Zhu", email = "jzhu@stat.wisc.edu", role = "aut"),
    person("Dipankar", "Bandyopadhyay", email = "bandyopd@gmail.com", role = "ctb"),
    person("Yincai", "Tang", email = "yctang@stat.ecnu.edu.cn", role= "ctb"))
Maintainer: Ting Fung Ma <tingfung@mailbox.sc.edu>
Description: Composite likelihood parameter estimate and asymptotic covariance matrix are calculated for the spatial ordinal data with replications, where spatial ordinal response with covariate and both spatial exponential covariance within subject and independent and identically distributed measurement error.  Parameter estimation can be performed by either solving the gradient function or maximizing composite log-likelihood. Parametric bootstrapping is used to estimate the Godambe information matrix and hence the asymptotic standard error and covariance matrix with parallel processing option. Moreover, the proposed surrogate residual, which extends the results of Liu and Zhang (2017) <doi: 10.1080/01621459.2017.1292915>, can act as a useful tool for model diagnostics.
License: GPL-2
Imports: pbivnorm (>= 0.6.0), MASS (>= 7.3-45), rootSolve (>= 1.7),
        parallel, doParallel (>= 1.0.11), foreach (>= 1.2.0),
        tmvmixnorm (>= 1.0.2), utils, stats
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-08 17:02:47 UTC; ralph
Repository: CRAN
Date/Publication: 2026-05-09 16:30:18 UTC
Author: Ting Fung Ma [cre, aut],
  Pingping Wang [aut],
  Jun Zhu [aut],
  Dipankar Bandyopadhyay [ctb],
  Yincai Tang [ctb]
Built: R 4.6.0; ; 2026-05-09 17:44:07 UTC; unix

Package: clordr
Type: Package
Title: Composite Likelihood Inference and Diagnostics for Replicated
        Spatial Ordinal Data
Version: 1.7.2
Authors@R: c(person("Ting Fung", "Ma", email = "tingfung@mailbox.sc.edu", role = c("cre", "aut")),
    person("Pingping", "Wang", email = "catherinewpp@126.com", role = "aut"),
    person("Jun", "Zhu", email = "jzhu@stat.wisc.edu", role = "aut"),
    person("Dipankar", "Bandyopadhyay", email = "bandyopd@gmail.com", role = "ctb"),
    person("Yincai", "Tang", email = "yctang@stat.ecnu.edu.cn", role= "ctb"))
Maintainer: Ting Fung Ma <tingfung@mailbox.sc.edu>
Description: Composite likelihood parameter estimate and asymptotic covariance matrix are calculated for the spatial ordinal data with replications, where spatial ordinal response with covariate and both spatial exponential covariance within subject and independent and identically distributed measurement error.  Parameter estimation can be performed by either solving the gradient function or maximizing composite log-likelihood. Parametric bootstrapping is used to estimate the Godambe information matrix and hence the asymptotic standard error and covariance matrix with parallel processing option. Moreover, the proposed surrogate residual, which extends the results of Liu and Zhang (2017) <doi: 10.1080/01621459.2017.1292915>, can act as a useful tool for model diagnostics.
License: GPL-2
Imports: pbivnorm (>= 0.6.0), MASS (>= 7.3-45), rootSolve (>= 1.7),
        parallel, doParallel (>= 1.0.11), foreach (>= 1.2.0),
        tmvmixnorm (>= 1.0.2), utils, stats
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-08 17:02:47 UTC; ralph
Repository: CRAN
Date/Publication: 2026-05-09 16:30:18 UTC
Author: Ting Fung Ma [cre, aut],
  Pingping Wang [aut],
  Jun Zhu [aut],
  Dipankar Bandyopadhyay [ctb],
  Yincai Tang [ctb]
Built: R 4.6.0; ; 2026-05-09 18:52:24 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cl)
export(cl.rord)
export(cl_l)
export(cle.rord)
export(clic)
export(sim.rord)
export(surrogate.residual)
import(MASS)
import(doParallel)
import(foreach)
import(parallel)
import(pbivnorm)
import(rootSolve)
import(stats)
import(tmvmixnorm)
import(utils)

#' Composite Likelihood Estimation for Replciations of Spatial Ordinal Data
#'
#' \code{cle.rord} Estimate parameters (including regression coefficient and cutoff) for replications of spatial ordinal data using pairwise likelihood approach.
#'
#' @param response a matrix of observation (row: spatial site and column: subject).
#' @param covar regression (design) matrix, including intercepts.
#' @param ini.sp initial estimate for spatial parameter, \eqn{\phi,\sigma^2} (default: c(0.5,0.5)).
#' @param location a matrix contains spatial location of sites within each subject.
#' @param radius radius for selecting pairs for the composite likelihood estimation.
#' @param n.sim number of simulation used for parametric bootstrapping (and hence used for asymptotic variance and standard error).
#' @param output logical flag indicates whether printing out result (default: \code{TRUE}).
#' @param SE logical flag for detailed output.
#' @param parallel logical flag indicates using parallel processing (default: \code{FALSE}).
#' @param n.core number of physical cores used for parallel processing (when \code{parallel} is \code{TRUE}, default value is \code{max(detectCores()/2,1)}).
#' @param est.method logical flag (default) \code{TRUE} for rootsolve and \code{FALSE} for L-BFGS-B.
#' @param maxiter maximum number of iterations in the root solving of gradient function (dafault: 100).
#' @param rtol relative error tolerrance  in the root solving of gradient function (default: 1e-6).
#' @param factr reduction in the objective (-logCL) within this factor of the machine tolerance for L-BFGS-B (default: 1e7).

#'
#' @details Given vector of ordinal responses, the design matrix, spatial location for sites, weight radius (for pair selection), and the prespecified number of simulation used for estimating the Godambe information matrix. Initial estimate is obtained by fitting model without spatial dependence (using \code{MASS::polr()}) and optional guess of spatial parameters. The function first estimates parameters of interest by either solving the gradient of composite log-likelihood using \code{rootSolve::multiroot()} or maximize the composite log-likelihood by \code{optim(..., method="L-BFGS-B")}. The asymptotic covariance matrix and standard error of parameters are then estimated by parametric boostrapping. Although the default root solving option is typically more efficient, it may encounter runtime error if negative value of \eqn{\phi} is evaluated (and L-BFGS-B approach should be used).
#' @return \code{cle.rord} returns a list contains:
#' @return \code{vec.par}: a vector of estimator for \eqn{\theta=(\alpha,\beta,\phi,\sigma^2)};
#'
#' @return \code{vec.se}: a vector of standard error for the estimator;
#' @return \code{mat.asyvar}: estimated asymptotic covariance matrix \eqn{H^{-1}(\theta)J(\theta)H^{-1}(\theta)} for the estimator;
#' @return \code{mat.Hessian}: Hessian matrix at the parameter estimate;
#' @return \code{mat.J}: Sensitivity matrix estimated by parametric boostrapping; and
#' @return \code{CLIC}: Composite likelihood information criterion (see help manual of \code{clic()} for detail).
#'
#' @import pbivnorm  MASS rootSolve parallel doParallel foreach tmvmixnorm utils stats
#' @export
#' @examples
#' set.seed(1228)
#' n.subject <- 20
#' n.lat <- n.lon <- 10
#' n.site <- n.lat*n.lon
#'
#' beta <- c(1,2,-1) # First 1 here is the intercept
#' midalpha <- c(1.15, 2.18) ; phi <- 0.6 ; sigma2 <- 0.7
#'
#' true <- c(midalpha,beta,phi,sigma2)
#'
#' Xi <- rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)
#'
#'  VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)
#'
#'  for(i in 1:n.subject){ for(j in 1:n.site){
#'      VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
#'        }
#'  }
#'
#' location <- cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
#' sim.data <- sim.rord(n.subject, n.site, n.rep = 2, midalpha, beta, phi, sigma2, covar=VV, location)
#'
#' \donttest{
#' options(digits=3)
#' result <- cle.rord(response=sim.data[[1]], covar=VV,
#'           location = location ,radius = 4, n.sim = 100, output = TRUE, parallel=TRUE, n.core =2)
#' result$vec.par
#' # alpha2  alpha3   beta0   beta1   beta2     phi sigma^2
#' # 1.249   2.319   1.169   1.990  -1.000   0.668   0.678
#'
#' result$vec.se
#' # alpha2  alpha3   beta0   beta1   beta2     phi sigma^2
#' # 0.0704  0.1201  0.1370  0.2272  0.0767  0.0346  0.1050
#'
#' }
#'

cle.rord <- function(response, covar, location ,radius=4, n.sim=100, output = TRUE, SE = TRUE,
                     parallel = FALSE, n.core = max(detectCores()/2,1),
                     ini.sp = c(0.5,0.5), est.method = TRUE, maxiter = 100, rtol = 1e-6, factr = 1e7) {

  n <- ncol(response) ; N <- nrow(response) ; J <- length(levels(factor(response))) ; p <- NCOL(covar)
  lt <- J + p

  i <- j <- 1

  d <- c(dist(location, method='euclidean')) # calculate the distance between each pair of sites

  wd <- which(d <= radius) # select pairs of sites to be included
  dwdv <- t(d[wd]) # corresponding distance of selected pair
  wn <- length(dwdv) # number of pairs
  cm <- combn(1:N,2)   # the combination of the pairs
  cmwdv <- cm[,wd] # the combination of the pairs included into the composite likelihood
  base <- diag(J+1)   # the identity matrix

  # Initial value
  po1 = MASS::polr(factor(as.vector(response) )~covar[,-1],method='probit')
  ini <- unname(c(po1$zeta[-1] - po1$zeta[1],-po1$zeta[1],po1$coef,ini.sp))


  func<-function(theta, response, covar){

    sum_func <- 0
    for(i in 1:n){
      y <- response[,i]
      X <- covar[((i-1)*N+1):(i*N),]

      sum_func <- sum_func+cl_l(theta,y,X,dwdv,cmwdv,lt,wn,base,J,p)
    }
    return(sum_func)
  }


  dfun <- function(theta, response, covar){

    if(theta[lt-1] < 0) stop("phi cannot be negative, please use est.method = FALSE for L-BFGS-B and/or change ini.sp!!!\n")

    sum_gr <- numeric(J+p)
    for(i in 1:n){
      y <- response[,i]
      X <- covar[((i-1)*N+1):(i*N),]

      sum_gr <- sum_gr+cl(theta,y,X,dwdv,cmwdv,lt,wn,base,J,p)$gr
    }

    return(sum_gr)
  }

  if (est.method) {
  vec.par <- unname(rootSolve::multiroot(f=dfun,start=ini,response=response,covar=covar,
                                         maxiter = maxiter, rtol = rtol)$root)
  } else {
    lower <- ini-abs(ini)*0.5
    upper <- ini+abs(ini)*0.2

    if (lower[lt-1]  < 0) lower[lt-1] <- 1e-4

    vec.par <- unname( optim(par=ini, fn=func, gr=dfun, lower=lower, upper=upper,
                 method = 'L-BFGS-B',
                 control = list(factr = factr),
                 response=response, covar=covar)$par )
  }


  if (output) {
    cat("Estimate for\n")
    cat("alpha:",round(vec.par[1:(J-2)],digits=3),"\n")
    cat("beta:",round(vec.par[(J-1):(J+p-2)],digits=3),"\n")
    cat("phi, sigma^2:",round(tail(vec.par,2),digits=3),"\n")

  }
  if (SE) {
  sim.tmp <- sim.rord(n.subject=n, n.site=N, n.rep=n.sim,
                      midalpha=vec.par[1:(J-2)], beta=vec.par[(J-1):(lt-2)], phi=vec.par[lt-1], sigma2=vec.par[lt],
                      covar, location)

  is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  {(abs(x - round(x)) < tol)&&(!(x== 0))}
  # Small function to check if positive integer

  if (parallel == TRUE){
    i <- 1
    options(warn=-1) # Remove warning message: closing unused connection

    if (!is.double(n.core)) {
      cat("Wrong input type for n.core, replaced by the default value, max(detectCores()/2,1) = ",max(parallel::detectCores()/2,1),"\n")
      n.core <- max(parallel::detectCores()/2,1)
    }else if(!is.wholenumber(n.core)) {
      cat("Input for n.core is not a positive integer, replaced by the default value, max(detectCores()/2,1) = ",max(parallel::detectCores()/2,1),"\n")
      n.core <- max(parallel::detectCores()/2,1)
    }

    cl_cluster <- parallel::makeCluster(n.core)
    doParallel::registerDoParallel(cl_cluster)
    mat.J <- cov(foreach(i = 1:n.sim, .export="cl", .combine=rbind) %dopar% {
      dfun(theta=vec.par, response=sim.tmp[[i]],covar=covar)
    })

    stopCluster(cl_cluster)

    options(warn=0) # Resume warning message

  } else{

  score.tmp <- matrix(0, ncol=lt, nrow=n.sim)

  for (j in 1:n.sim) {
    response.tmp <- sim.tmp[[j]]
    score.tmp[j,] <- dfun(theta = vec.par, response = response.tmp, covar = covar)
  }

  mat.J <- cov(score.tmp)

  }

  if (output) cat("Completed calculation of variability matrix, Now start to calculate Hessian matrix.\n")

  func.outsq <- function(vec) vec %o% vec

  mat.H <- matrix(0,nrow=length(vec.par), ncol=length(vec.par))

  for (i in 1:n) {
    y <- response[,i]
    X <- covar[((i-1)*N+1):(i*N),]

    mat.score.i <- cl(theta=vec.par,y,X,dwdv,cmwdv,lt,wn,base,J,p)$grmat[,c(3:J,(J+2):(lt+3))]


    mat.H <- mat.H + matrix(rowSums(apply(mat.score.i, 1, func.outsq)), nrow = length(vec.par),
                                        byrow = TRUE)/wn

  }


  mat.H.inv <- solve(mat.H)



  mat.asyvar <- mat.H.inv %*% mat.J %*% mat.H.inv

  vec.se <- sqrt(diag(mat.asyvar))
  CLIC <- clic(logCL= - func(theta=vec.par,response = response, covar=covar),
               mat.hessian = mat.H,mat.J = mat.J)
}

  names(vec.par) <- c(paste0(rep("alpha",J-2),2:(J-1)),paste0(rep("beta",p),0:(p-1)),"phi", "sigma^2")

if (SE) {
  names(vec.se) <- colnames(mat.asyvar) <- names(vec.par)

    ans <- list(vec.par=vec.par, vec.se=vec.se, mat.asyvar=mat.asyvar, mat.Hessian=mat.H, mat.J=mat.J, CLIC=CLIC)
  } else {

    ans <-  list(vec.par=vec.par)
  }
ans
}

#' Surrogate Residuals for Replciations of Spatial Ordinal Data
#'
#' \code{surrogate.residual} simulate the surrogate residual with the the given parameter value and covariate for model diagnostics.
#'
#' @param response a matrix of observation (row: spatial site and column: subject).
#' @param covar regression (design) matrix, including intercepts.
#' @param location a matrix contains spatial location of sites within each subject.
#' @param seed seed input for simulation (default =\code{NULL}).
#' Parameter values:
#' @param midalpha cutoff for latent ordinal response.
#' @param beta regression coefficient  for \code{covar}.
#'
#' @param sigma2 \eqn{\sigma^2} for exponential covariance.
#' @param phi spatial correlation for exponential covariance.
#' @param burn.in burn-in length (i.e. declaring the initial sample).
#' @param output logical flag indicates whether printing out result (default: \code{TRUE}).
#'
#' @details Given vector of observed responses, the design matrix, spatial location for sites and parameter value, raw surrogate residuals are simulated using an efficient Gibbs sampling, which can be used for model diagnostics. When the fitted model is correct, the raw surrogate residuals among subjects should follow multivariate normal with mean 0 and covariance Sigma. If the model is correct, residual plot should be close to a null plot or random scatter. For example, it can be used to check the potential missing in covariate, non-linearity of covariate and outliers. In particular for the example below, the residual plot shows that linearity of Xi is adequate for the model.
#'
#' @return \code{surrogate.residual} returns a (no. spatial site * no. subject) matrix contains
#' raw surrogate residuals with element corresponds to the response matrix.
#'
#' @import pbivnorm  MASS rootSolve parallel doParallel foreach tmvmixnorm utils stats
#' @export
#'
#' @examples
#' set.seed(1228)
#' n.subject <- 50
#' n.lat <- n.lon <- 10
#' n.site <- n.lat*n.lon
#'
#' beta <- c(1,2,-1) # First 1 here is the intercept
#' midalpha <- c(1.15, 2.18) ; phi <- 0.6 ; sigma2 <- 0.7
#'
#' true <- c(midalpha,beta,phi,sigma2)
#'
#' Xi <- rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)
#'
#'  VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)
#'
#'  for(i in 1:n.subject){ for(j in 1:n.site){
#'      VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
#'        }
#'  }
#'
#' location <- cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
#' response <- sim.rord(n.subject, n.site, n.rep = 1,
#' midalpha, beta, phi, sigma2, covar=VV, location)[[1]]
#'
#' \donttest{
#' # Example for linearity of covariate
#' sur.resid <- surrogate.residual(response, covar=VV, location, seed =1,
#' midalpha, beta, sigma2, phi,
#' burn.in=20, output = TRUE)
#'
#' scatter.smooth(rep(Xi,each=n.site),c(sur.resid),
#' main="Surrogate residual against Xi", xlab="Xi", ylab="Surrogate residual",
#' lpars = list(col = "red", lwd = 3, lty = 2))
#'
#' abline(h=0, col="blue")
#' }
#'

surrogate.residual <- function(response, covar, location, seed = NULL, midalpha, beta, sigma2, phi,
                               burn.in=20, output = TRUE) {

  # n: n.subject
  # N: number of sites for each subject
  n <- ncol(response) ; N <- nrow(response)

  residual <- matrix(NA, nrow = N, ncol = n)

  talpha1 <- c(-Inf, 0, midalpha, Inf)
  J <- length(talpha1)
  start.value <-  (talpha1[1:(J-1)] + talpha1[2:J])/2 ;  start.value[1] <- talpha1[2] - 0.1 ; start.value[J-1] <-  talpha1[J-1] + 0.1


  L2matrix = as.matrix(dist(location, method='euclidean', upper=TRUE, diag=TRUE))
  SIGMA = sigma2*(1+phi*L2matrix)*exp(-phi*L2matrix)

  set.seed(seed)

  for(i in 1:n){
    y.tmp <- response[,i] ; X.tmp <- covar[((i-1)*N+1):(i*N),]

    lower.tmp <- talpha1[y.tmp +1] ; upper.tmp <- talpha1[y.tmp + 2]
    start.tmp <- start.value[y.tmp+1]



    residual[,i] <- tmvmixnorm::rtmvn(n=1, Mean=c(X.tmp%*%beta), Sigma=SIGMA, D = diag(1, N),
                                      lower=lower.tmp, upper=upper.tmp, int=start.tmp,burn = burn.in, thin = 0) - c(X.tmp%*%beta)

      #c(tmvtnorm::rtmvnorm(n=1, mean=c(X.tmp%*%beta), sigma=SIGMA,
      #                                   lower=lower.tmp, upper=upper.tmp, algorithm="gibbs",
      #                                   start.value=start.tmp, burn.in.samples = burn.in)) - c(X.tmp%*%beta)

    if (output) {
      cat("Surrogate residual generated:",i,"out of", n,"\r")
      flush.console()
    }

  }

  if (output) cat("\b\n")

  residual
}

#' Composite Likelihood Calculation for Replciations of Spatial Ordinal Data (for illustration)
#'
#' \code{cl.rord} Calculate the negative composite log-likelihood value for replications of spatial ordinal data at given value of parameter value.
#' Note that this function is not directly used in \code{cle.rord} but illustration only.
#'
#' @param theta a vector of parameter value
#' @param response a matrix of observation (row: spatial site and column: subject).
#' @param covar regression (design) matrix, including intercepts.
#' @param location a matrix contains spatial location of sites within each subject
#' @param radius radius for selecting pairs for the composite likelihood estimation.
#'
#' @return \code{cl.rord} returns a list: negative composite log-likelihood, a vector of first-order partial derivatives for \code{theta}.
#'
#' @import pbivnorm  MASS rootSolve parallel doParallel foreach tmvmixnorm utils stats
#' @export
#' @examples
#' set.seed(1203)
#' n.subject <- 10
#' n.lat <- n.lon <- 10
#' n.site <- n.lat*n.lon
#'
#' beta <- c(1,2,-1) # First 1 here is the intercept
#' midalpha <- c(1.15, 2.18) ; sigma2 <- 0.7 ; phi <- 0.8
#'
#' true = c(midalpha,beta,sigma2,phi)
#'
#' Xi = rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)
#'
#'  VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)
#'
#'  for(i in 1:n.subject){ for(j in 1:n.site){
#'      VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
#'        }
#'  }
#'
#' location = cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
#' sim.data <- sim.rord(n.subject, n.site, n.rep = 2, midalpha, beta, sigma2, phi, covar=VV, location)
#'
#' cl.rord(theta=true,response=sim.data[[1]], covar=VV, location, radius = 4)
#'

cl.rord <- function(theta,response, covar, location ,radius = 4) {

  n <- ncol(response) ; N <- nrow(response) ; J <- length(levels(factor(response))) ; p <- NCOL(covar)

  lt <- J + p

  d <- c(dist(location, method='euclidean')) # calculate the distance between each pair of sites

  wd <- which(d <= radius) # select pairs of sites to be included
  dwdv <- t(d[wd]) # corresponding distance of selected pair
  wn <- length(dwdv) # number of pairs
  cm <- combn(1:N,2)   # the combination of the pairs
  cmwdv <- cm[,wd] # the combination of the pairs included into the composite likelihood
  base <- diag(J+1)   # the identity matrix

  # the negative composite log-likelihood value
  func<-function(theta, response, covar){
    sum_func <- 0
    for(i in 1:n){
      y <- response[,i]
      X <- covar[((i-1)*N+1):(i*N),]

      sum_func <- sum_func+cl_l(theta,y,X,dwdv,cmwdv,lt,wn,base,J,p)
    }
    return(sum_func)
  }

  # the derivative of the negative composite log-likelihood value
  dfun<-function(theta, response, covar){
    sum_gr <- numeric(J+p)
    for(i in 1:n){
      y <- response[,i]
      X <- covar[((i-1)*N+1):(i*N),]

      sum_gr <- sum_gr+cl(theta,y,X,dwdv,cmwdv,lt,wn,base,J,p)$gr
    }
    return(unname(sum_gr))
  }

  sum_func <- func(theta, response, covar)
  sum_gr <- dfun(theta, response, covar)

list(func=sum_func,score=sum_gr)
}

#' Simulation of Replciations of Spatial Ordinal Data
#'
#' \code{sim.rord} Simulate replications of spatial ordinal data
#'
#' @param n.subject number of subjects.
#' @param n.site number of spatial sites for each subject.
#' @param n.rep number of simulation.
#' Parameter inputs include:
#' @param midalpha cutoff parameter (excluding -Inf and +Inf);
#' @param beta regression coefficient;
#' @param phi dependence parameter for spatial dependence; and
#' @param sigma2 sigma^2 (variance) for the spatial dependence.
#' @param covar regression (design) matrix, including intercepts.
#' @param location a matrix contains spatial location of sites within each subject.
#'
#' @return \code{sim.rord} returns a list (length \code{n.rep}) of matrix (\code{n.subject*n.site}) with the underlying parameter as inputs.
#'
#' @import pbivnorm  MASS rootSolve parallel doParallel foreach tmvmixnorm utils stats
#'
#' @export
#'
#' @examples
#' set.seed(1203)
#' n.subject <- 100
#' n.lat <- n.lon <- 10
#' n.site <- n.lat*n.lon
#'
#' beta <- c(1,2,-1) # First 1 here is the intercept
#' midalpha <- c(1.15, 2.18) ; phi <- 0.8 ; sigma2 <- 0.7
#'
#' true <- c(midalpha,beta,sigma2,phi)
#'
#' Xi <- rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)
#'
#'  VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)
#'
#'  for(i in 1:n.subject){ for(j in 1:n.site){
#'      VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
#'        }
#'  }
#'
#' location <- cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
#' sim.data <- sim.rord(n.subject, n.site, n.rep = 2, midalpha, beta, phi, sigma2, covar=VV, location)
#'
#' length(sim.data)
#' head(sim.data[[1]])
#' dim(sim.data[[1]])
#' hist(sim.data[[1]])



sim.rord <- function(n.subject, n.site, n.rep = 100, midalpha, beta, phi, sigma2, covar, location) {

  N <- n.site ; n <- n.subject

  observe<-function(x,alpha) {
    b = rep(0,length(x))
    I = length(alpha)
    alpha=c(0,alpha)
    for(i in 1:I){
      b[which(x>=alpha[i] & x<alpha[i+1])]=i
    }
    b[which(x>=alpha[i+1])]=I+1
    return(b)
  }

  talpha1 = c(-Inf, 0, midalpha, Inf)
  true = c(midalpha,beta,phi,sigma2)
  lt = length(true)


  L2matrix = as.matrix(dist(location, method='euclidean', upper=TRUE, diag=TRUE))
  SIGMA = sigma2*(1+phi*L2matrix)*exp(-phi*L2matrix)


  p <- NCOL(covar) ; J <- length(talpha1)

  R <- chol(SIGMA)

  ystar_gen <- function(x, R=R, beta=beta, sigma2=sigma2){
    zstar <-  t(R)%*%array(rnorm(N),c(N,1))+ covar[((x-1)*N+1):(x*N),]%*%beta
    epstar <- diag(sqrt(1-sigma2),N)%*%array(rnorm(N),c(N,1))
    return(ystar = epstar+zstar)
  }

  result <- NULL

  i <- j <- 1

  for (j in 1:n.rep) {

  ystar <- matrix(0, N, n)

    for(i in 1:n){ ystar[,i] <- ystar_gen(i, R=R, beta=beta, sigma2=sigma2)  }

    yn <- apply(ystar,2,observe,midalpha) # Convert to ordinal

    result <- c(result,list(yn))
  }

  result
}

#' Composite likelihood Information Criterion
#'
#' \code{clic} Calculating the Composite likelihood information criterion proposed by Varin and Vidoni (2005)
#'
#' @return \code{CLIC}: Composite likelihood information criterion proposed by Varin and Vidoni (2005)
#' @param logCL value of composite log-likelihood.
#' @param mat.hessian hessian matrix.
#' @param mat.J Sensitivity matrix
#'
#' @return \code{clic}: Composite likelihood information criterion proposed by Varin and Vidoni (2005)
#'
#' @details Varin and Vidoni (2005) proposed the information criterion in the form:
#' \eqn{-2*logCL(theta) + 2*trace(H^{-1}(\theta)J(\theta))}
#'
#' @references Varin, C. and Vidoni, P. (2005) A note on composite likelihood inference and model selection. \emph{Biometrika} 92: 519--528.
#'
#' @import pbivnorm  MASS rootSolve parallel doParallel foreach tmvmixnorm utils stats
#' @export

clic <- function(logCL,mat.hessian,mat.J){
  2*(-logCL + sum(diag(solve(mat.hessian)%*%mat.J)))
}

#' Composite Likelihood Calculation for Spatial Ordinal Data without Replications (for implmentation)
#'
#' @param x an object to be merged into list of object.
#' @param y an object to be merged into list.
#' @param mergeUnnamed select an element if it has a.) an empty name in y and
#' mergeUnnamed is true or  b.) a name _not_ contained in x
#' @return \code{merge.list} returns a list: a merged list


merge.list <- function(x, y=NULL, mergeUnnamed=TRUE) {
  if (is.null(y)) {
    return(as.list(x))
  } else {
    y <- as.list(y)

    # replace NA in names with ""
    if (!is.null(names(x))) {
      names(x)[is.na(names(x))] <- ""
    }
    if (!is.null(names(y))) {
      names(y)[is.na(names(y))] <- ""
    }

    names1 <- names(x)
    names2 <- names(y)

    # select an element if it has:
    # a.) an empty name in y _and_
    #     mergeUnnamed is true _or_
    # b.) a name _not_ contained in x
    isUnique <- if (mergeUnnamed) {
      if (is.null(names2) | is.null(names1)) {
        rep(TRUE, length(y))
      } else {
        (nchar(names2) == 0) | !(names2 %in% names1)
      }
    } else {
      !(names2 %in% names1) & (nchar(names2) != 0)
    }
    mergedList <- c(x, y[isUnique])
    return(mergedList)
  }
}

#' Composite Likelihood Calculation for Spatial Ordinal Data without Replications (for implmentation)
#'
#' \code{cl} Calculate the negative composite log-likelihood value and score function for a particular subject given parameter value and other input variables.
#'
#' @param theta a vector of parameter value.
#' @param y a vector of observation for the subject.
#' @param X covariate for the particular subject.
#' @param dwdv corresponding distance of selected pair.
#' @param cmwdv combination of the pairs included into the composite likelihood.
#' @param lt number of parameter (i.e. length of theta).
#' @param wn number of pairs with distance.
#' @param base identity matrix with dimension \code{J+1}.
#' @param J number of category among (ALL) observed response.
#' @param p number of covariate (i.e. number of column of \code{X}).
#'
#' @return \code{cl} returns a list: composite log-likelihood value and a vector of first-order partial derivatives for \code{theta}.
#'
#' @import pbivnorm  MASS rootSolve parallel doParallel foreach tmvmixnorm utils stats
#' @export


cl<-function(theta,y,X,dwdv,cmwdv,lt,wn,base,J,p) {

  xi<-function(u1,u2,r,sqr) (u2-r*u1)/sqr

  Xa = X[cmwdv[1,],]
  Xb = X[cmwdv[2,],]

  pairy = matrix(y[cmwdv],nrow=2)
  b = c(-10^4,0,theta[1:(J-2)],10^4,theta[(J-1):(lt-2)])
  Idx=A1lo=A2lo=A1hi=A2hi=list()

  for (j in 0:(J-1)) {
    for (k in 0:(J-1)) {
      A1lo = merge.list(A1lo,list(base[j+1,]))
      A1hi = merge.list(A1hi,list(base[j+2,]))
      Idx = merge.list(Idx, list(which(pairy[1,]==j&pairy[2,]==k)))##among those neighbour pairs
      A2lo = merge.list(A2lo,list(base[k+1,]))
      A2hi = merge.list(A2hi, list(base[k+2,]))
    }
  }
  Ldx = rapply(Idx,function(x) length(x),how="list")
  Dwdv2 = rapply(Idx,function(x) dwdv[x],how="unlist")
  X1 = rapply(Idx, function(x) Xa[x,],how="list")
  X2 = rapply(Idx, function(x) Xb[x,],how="list")

  xh <- function(x,y,z) matrix(c(rep(x, each = y),-z), byrow=F,nrow=y)
  X1lo0 = mapply(xh,A1lo,Ldx,X1)
  X1lo <- matrix(NA,0,(J+p+1))
  for( g in 1:(J^2) ){
    if(!length(X1lo0[[g]])==0){
      X1lo <- rbind(X1lo, X1lo0[[g]])
    }
  }
  #X1lo <-do.call(rbind,X1lo0)
  X1hi0 = mapply(xh,A1hi,Ldx,X1)
  X1hi <- matrix(NA,0,(J+p+1))
  for( g in 1:(J^2) ){
    if(!length(X1hi0[[g]])==0){
      X1hi <- rbind(X1hi, X1hi0[[g]])
    }
  }
  #X1hi <- do.call(rbind,X1hi0)
  X2lo0 = mapply(xh,A2lo,Ldx,X2)
  X2lo <- matrix(NA,0,(J+p+1))
  for( g in 1:(J^2) ){
    if(!length(X2lo0[[g]])==0){
      X2lo <- rbind(X2lo, X2lo0[[g]])
    }
  }
  #X2lo <- do.call(rbind,X2lo0)
  X2hi0 = mapply(xh,A2hi,Ldx,X2)
  X2hi <- matrix(NA,0,(J+p+1))
  for( g in 1:(J^2) ){
    if(!length(X2hi0[[g]])==0){
      X2hi <- rbind(X2hi, X2hi0[[g]])
    }
  }
  ### b contains all alpha and beta; alpha0=-10^4; alphaJ = 10^4###
  Mu2hh = cbind(X1hi%*%b,X2hi%*%b)
  Mu2ll = cbind(X1lo%*%b,X2lo%*%b)
  Mu2hl= cbind(X1hi%*%b,X2lo%*%b)
  Mu2lh = cbind(X1lo%*%b,X2hi%*%b)
  rr =  theta[lt]*(1+theta[lt-1]*Dwdv2)*exp(-theta[lt-1]*Dwdv2)
  de = 1-rr^2
  sde = sqrt(de)
  re2 = pbivnorm::pbivnorm(Mu2hh,rho=rr) + pbivnorm::pbivnorm(Mu2ll,rho=rr) -
    (pbivnorm::pbivnorm(Mu2hl,rho=rr)+pbivnorm::pbivnorm(Mu2lh,rho=rr))
  if(sum(is.na(re2))!=0){
    re2[which(is.na(re2))] <- 0
  }
  if(sum(re2)!=0){
    re2[which(re2<=0)]=1e-99 } else re2 <- rep(1e-99, length(re2))  #cl vector value & denominator; length inx

  numer12 = (c(dnorm(Mu2hh[,1])*pnorm(xi(Mu2hh[,1],Mu2hh[,2], rr, sde)))*X1hi+
               c(dnorm(Mu2hh[,2])*pnorm(xi(Mu2hh[,2],Mu2hh[,1], rr, sde)))*X2hi) +
    (c(dnorm(Mu2ll[,1])*pnorm(xi(Mu2ll[,1],Mu2ll[,2], rr, sde)))*X1lo+
       c(dnorm(Mu2ll[,2])*pnorm(xi(Mu2ll[,2],Mu2ll[,1], rr, sde)))*X2lo) -
    (c(dnorm(Mu2hl[,1])*pnorm(xi(Mu2hl[,1],Mu2hl[,2], rr, sde)))*X1hi+
       c(dnorm(Mu2hl[,2])*pnorm(xi(Mu2hl[,2],Mu2hl[,1], rr, sde)))*X2lo) -
    (c(dnorm(Mu2lh[,1])*pnorm(xi(Mu2lh[,1],Mu2lh[,2], rr, sde)))*X1lo+
       c(dnorm(Mu2lh[,2])*pnorm(xi(Mu2lh[,2],Mu2lh[,1], rr, sde)))*X2hi)

  dense = (1/(2*pi*sde))*{
    exp(-0.5*(rowSums(Mu2hh^2)-2*rr*Mu2hh[,1]*Mu2hh[,2])/de) +
      exp(-0.5*(rowSums(Mu2ll^2)-2*rr*Mu2ll[,1]*Mu2ll[,2])/de) -
      exp(-0.5*(rowSums(Mu2hl^2)-2*rr*Mu2hl[,1]*Mu2hl[,2])/de) -
      exp(-0.5*(rowSums(Mu2lh^2)-2*rr*Mu2lh[,1]*Mu2lh[,2])/de)
  }
  numer3 =  c(dense*(1+theta[lt-1]*Dwdv2)*exp(-theta[lt-1]*Dwdv2) )
  numer4 = c(-dense*theta[lt]*Dwdv2*Dwdv2*theta[lt-1]*exp(-theta[lt-1]*Dwdv2))
  grmat2 = cbind(numer12,numer3,numer4)/re2
  f2 = -sum(log(re2))/wn
  gr2 = -colSums(grmat2[,c(3:J,(J+2):(lt+3))])/wn
  return(list(func = f2, gr = gr2, grmat = grmat2)) #change
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cle.rord.R
\name{cle.rord}
\alias{cle.rord}
\title{Composite Likelihood Estimation for Replciations of Spatial Ordinal Data}
\usage{
cle.rord(
  response,
  covar,
  location,
  radius = 4,
  n.sim = 100,
  output = TRUE,
  SE = TRUE,
  parallel = FALSE,
  n.core = max(detectCores()/2, 1),
  ini.sp = c(0.5, 0.5),
  est.method = TRUE,
  maxiter = 100,
  rtol = 1e-06,
  factr = 1e+07
)
}
\arguments{
\item{response}{a matrix of observation (row: spatial site and column: subject).}

\item{covar}{regression (design) matrix, including intercepts.}

\item{location}{a matrix contains spatial location of sites within each subject.}

\item{radius}{radius for selecting pairs for the composite likelihood estimation.}

\item{n.sim}{number of simulation used for parametric bootstrapping (and hence used for asymptotic variance and standard error).}

\item{output}{logical flag indicates whether printing out result (default: \code{TRUE}).}

\item{SE}{logical flag for detailed output.}

\item{parallel}{logical flag indicates using parallel processing (default: \code{FALSE}).}

\item{n.core}{number of physical cores used for parallel processing (when \code{parallel} is \code{TRUE}, default value is \code{max(detectCores()/2,1)}).}

\item{ini.sp}{initial estimate for spatial parameter, \eqn{\phi,\sigma^2} (default: c(0.5,0.5)).}

\item{est.method}{logical flag (default) \code{TRUE} for rootsolve and \code{FALSE} for L-BFGS-B.}

\item{maxiter}{maximum number of iterations in the root solving of gradient function (dafault: 100).}

\item{rtol}{relative error tolerrance  in the root solving of gradient function (default: 1e-6).}

\item{factr}{reduction in the objective (-logCL) within this factor of the machine tolerance for L-BFGS-B (default: 1e7).}
}
\value{
\code{cle.rord} returns a list contains:

\code{vec.par}: a vector of estimator for \eqn{\theta=(\alpha,\beta,\phi,\sigma^2)};

\code{vec.se}: a vector of standard error for the estimator;

\code{mat.asyvar}: estimated asymptotic covariance matrix \eqn{H^{-1}(\theta)J(\theta)H^{-1}(\theta)} for the estimator;

\code{mat.Hessian}: Hessian matrix at the parameter estimate;

\code{mat.J}: Sensitivity matrix estimated by parametric boostrapping; and

\code{CLIC}: Composite likelihood information criterion (see help manual of \code{clic()} for detail).
}
\description{
\code{cle.rord} Estimate parameters (including regression coefficient and cutoff) for replications of spatial ordinal data using pairwise likelihood approach.
}
\details{
Given vector of ordinal responses, the design matrix, spatial location for sites, weight radius (for pair selection), and the prespecified number of simulation used for estimating the Godambe information matrix. Initial estimate is obtained by fitting model without spatial dependence (using \code{MASS::polr()}) and optional guess of spatial parameters. The function first estimates parameters of interest by either solving the gradient of composite log-likelihood using \code{rootSolve::multiroot()} or maximize the composite log-likelihood by \code{optim(..., method="L-BFGS-B")}. The asymptotic covariance matrix and standard error of parameters are then estimated by parametric boostrapping. Although the default root solving option is typically more efficient, it may encounter runtime error if negative value of \eqn{\phi} is evaluated (and L-BFGS-B approach should be used).
}
\examples{
set.seed(1228)
n.subject <- 20
n.lat <- n.lon <- 10
n.site <- n.lat*n.lon

beta <- c(1,2,-1) # First 1 here is the intercept
midalpha <- c(1.15, 2.18) ; phi <- 0.6 ; sigma2 <- 0.7

true <- c(midalpha,beta,phi,sigma2)

Xi <- rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)

 VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)

 for(i in 1:n.subject){ for(j in 1:n.site){
     VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
       }
 }

location <- cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
sim.data <- sim.rord(n.subject, n.site, n.rep = 2, midalpha, beta, phi, sigma2, covar=VV, location)

\donttest{
options(digits=3)
result <- cle.rord(response=sim.data[[1]], covar=VV,
          location = location ,radius = 4, n.sim = 100, output = TRUE, parallel=TRUE, n.core =2)
result$vec.par
# alpha2  alpha3   beta0   beta1   beta2     phi sigma^2
# 1.249   2.319   1.169   1.990  -1.000   0.668   0.678

result$vec.se
# alpha2  alpha3   beta0   beta1   beta2     phi sigma^2
# 0.0704  0.1201  0.1370  0.2272  0.0767  0.0346  0.1050

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clic.R
\name{clic}
\alias{clic}
\title{Composite likelihood Information Criterion}
\usage{
clic(logCL, mat.hessian, mat.J)
}
\arguments{
\item{logCL}{value of composite log-likelihood.}

\item{mat.hessian}{hessian matrix.}

\item{mat.J}{Sensitivity matrix}
}
\value{
\code{CLIC}: Composite likelihood information criterion proposed by Varin and Vidoni (2005)

\code{clic}: Composite likelihood information criterion proposed by Varin and Vidoni (2005)
}
\description{
\code{clic} Calculating the Composite likelihood information criterion proposed by Varin and Vidoni (2005)
}
\details{
Varin and Vidoni (2005) proposed the information criterion in the form:
\eqn{-2*logCL(theta) + 2*trace(H^{-1}(\theta)J(\theta))}
}
\references{
Varin, C. and Vidoni, P. (2005) A note on composite likelihood inference and model selection. \emph{Biometrika} 92: 519--528.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merged.list.R
\name{merge.list}
\alias{merge.list}
\title{Composite Likelihood Calculation for Spatial Ordinal Data without Replications (for implmentation)}
\usage{
\method{merge}{list}(x, y = NULL, mergeUnnamed = TRUE)
}
\arguments{
\item{x}{an object to be merged into list of object.}

\item{y}{an object to be merged into list.}

\item{mergeUnnamed}{select an element if it has a.) an empty name in y and
mergeUnnamed is true or  b.) a name _not_ contained in x}
}
\value{
\code{merge.list} returns a list: a merged list
}
\description{
Composite Likelihood Calculation for Spatial Ordinal Data without Replications (for implmentation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.rord.R
\name{sim.rord}
\alias{sim.rord}
\title{Simulation of Replciations of Spatial Ordinal Data}
\usage{
sim.rord(
  n.subject,
  n.site,
  n.rep = 100,
  midalpha,
  beta,
  phi,
  sigma2,
  covar,
  location
)
}
\arguments{
\item{n.subject}{number of subjects.}

\item{n.site}{number of spatial sites for each subject.}

\item{n.rep}{number of simulation.
Parameter inputs include:}

\item{midalpha}{cutoff parameter (excluding -Inf and +Inf);}

\item{beta}{regression coefficient;}

\item{phi}{dependence parameter for spatial dependence; and}

\item{sigma2}{sigma^2 (variance) for the spatial dependence.}

\item{covar}{regression (design) matrix, including intercepts.}

\item{location}{a matrix contains spatial location of sites within each subject.}
}
\value{
\code{sim.rord} returns a list (length \code{n.rep}) of matrix (\code{n.subject*n.site}) with the underlying parameter as inputs.
}
\description{
\code{sim.rord} Simulate replications of spatial ordinal data
}
\examples{
set.seed(1203)
n.subject <- 100
n.lat <- n.lon <- 10
n.site <- n.lat*n.lon

beta <- c(1,2,-1) # First 1 here is the intercept
midalpha <- c(1.15, 2.18) ; phi <- 0.8 ; sigma2 <- 0.7

true <- c(midalpha,beta,sigma2,phi)

Xi <- rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)

 VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)

 for(i in 1:n.subject){ for(j in 1:n.site){
     VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
       }
 }

location <- cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
sim.data <- sim.rord(n.subject, n.site, n.rep = 2, midalpha, beta, phi, sigma2, covar=VV, location)

length(sim.data)
head(sim.data[[1]])
dim(sim.data[[1]])
hist(sim.data[[1]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cl.R
\name{cl}
\alias{cl}
\title{Composite Likelihood Calculation for Spatial Ordinal Data without Replications (for implmentation)}
\usage{
cl(theta, y, X, dwdv, cmwdv, lt, wn, base, J, p)
}
\arguments{
\item{theta}{a vector of parameter value.}

\item{y}{a vector of observation for the subject.}

\item{X}{covariate for the particular subject.}

\item{dwdv}{corresponding distance of selected pair.}

\item{cmwdv}{combination of the pairs included into the composite likelihood.}

\item{lt}{number of parameter (i.e. length of theta).}

\item{wn}{number of pairs with distance.}

\item{base}{identity matrix with dimension \code{J+1}.}

\item{J}{number of category among (ALL) observed response.}

\item{p}{number of covariate (i.e. number of column of \code{X}).}
}
\value{
\code{cl} returns a list: composite log-likelihood value and a vector of first-order partial derivatives for \code{theta}.
}
\description{
\code{cl} Calculate the negative composite log-likelihood value and score function for a particular subject given parameter value and other input variables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cl.rord.R
\name{cl.rord}
\alias{cl.rord}
\title{Composite Likelihood Calculation for Replciations of Spatial Ordinal Data (for illustration)}
\usage{
cl.rord(theta, response, covar, location, radius = 4)
}
\arguments{
\item{theta}{a vector of parameter value}

\item{response}{a matrix of observation (row: spatial site and column: subject).}

\item{covar}{regression (design) matrix, including intercepts.}

\item{location}{a matrix contains spatial location of sites within each subject}

\item{radius}{radius for selecting pairs for the composite likelihood estimation.}
}
\value{
\code{cl.rord} returns a list: negative composite log-likelihood, a vector of first-order partial derivatives for \code{theta}.
}
\description{
\code{cl.rord} Calculate the negative composite log-likelihood value for replications of spatial ordinal data at given value of parameter value.
Note that this function is not directly used in \code{cle.rord} but illustration only.
}
\examples{
set.seed(1203)
n.subject <- 10
n.lat <- n.lon <- 10
n.site <- n.lat*n.lon

beta <- c(1,2,-1) # First 1 here is the intercept
midalpha <- c(1.15, 2.18) ; sigma2 <- 0.7 ; phi <- 0.8

true = c(midalpha,beta,sigma2,phi)

Xi = rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)

 VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)

 for(i in 1:n.subject){ for(j in 1:n.site){
     VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
       }
 }

location = cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
sim.data <- sim.rord(n.subject, n.site, n.rep = 2, midalpha, beta, sigma2, phi, covar=VV, location)

cl.rord(theta=true,response=sim.data[[1]], covar=VV, location, radius = 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sur_residual.R
\name{surrogate.residual}
\alias{surrogate.residual}
\title{Surrogate Residuals for Replciations of Spatial Ordinal Data}
\usage{
surrogate.residual(
  response,
  covar,
  location,
  seed = NULL,
  midalpha,
  beta,
  sigma2,
  phi,
  burn.in = 20,
  output = TRUE
)
}
\arguments{
\item{response}{a matrix of observation (row: spatial site and column: subject).}

\item{covar}{regression (design) matrix, including intercepts.}

\item{location}{a matrix contains spatial location of sites within each subject.}

\item{seed}{seed input for simulation (default =\code{NULL}).
Parameter values:}

\item{midalpha}{cutoff for latent ordinal response.}

\item{beta}{regression coefficient  for \code{covar}.}

\item{sigma2}{\eqn{\sigma^2} for exponential covariance.}

\item{phi}{spatial correlation for exponential covariance.}

\item{burn.in}{burn-in length (i.e. declaring the initial sample).}

\item{output}{logical flag indicates whether printing out result (default: \code{TRUE}).}
}
\value{
\code{surrogate.residual} returns a (no. spatial site * no. subject) matrix contains
raw surrogate residuals with element corresponds to the response matrix.
}
\description{
\code{surrogate.residual} simulate the surrogate residual with the the given parameter value and covariate for model diagnostics.
}
\details{
Given vector of observed responses, the design matrix, spatial location for sites and parameter value, raw surrogate residuals are simulated using an efficient Gibbs sampling, which can be used for model diagnostics. When the fitted model is correct, the raw surrogate residuals among subjects should follow multivariate normal with mean 0 and covariance Sigma. If the model is correct, residual plot should be close to a null plot or random scatter. For example, it can be used to check the potential missing in covariate, non-linearity of covariate and outliers. In particular for the example below, the residual plot shows that linearity of Xi is adequate for the model.
}
\examples{
set.seed(1228)
n.subject <- 50
n.lat <- n.lon <- 10
n.site <- n.lat*n.lon

beta <- c(1,2,-1) # First 1 here is the intercept
midalpha <- c(1.15, 2.18) ; phi <- 0.6 ; sigma2 <- 0.7

true <- c(midalpha,beta,phi,sigma2)

Xi <- rnorm(n.subject,0,1) ; Xj <- rbinom(n.site,1,0.6)

 VV <- matrix(NA, nrow = n.subject*n.site, ncol = 3)

 for(i in 1:n.subject){ for(j in 1:n.site){
     VV[(i-1)*n.site+j,] <- c(1,Xi[i],Xj[j])
       }
 }

location <- cbind(rep(seq(1,n.lat,length=n.lat),n.lat),rep(1:n.lon, each=n.lon))
response <- sim.rord(n.subject, n.site, n.rep = 1,
midalpha, beta, phi, sigma2, covar=VV, location)[[1]]

\donttest{
# Example for linearity of covariate
sur.resid <- surrogate.residual(response, covar=VV, location, seed =1,
midalpha, beta, sigma2, phi,
burn.in=20, output = TRUE)

scatter.smooth(rep(Xi,each=n.site),c(sur.resid),
main="Surrogate residual against Xi", xlab="Xi", ylab="Surrogate residual",
lpars = list(col = "red", lwd = 3, lty = 2))

abline(h=0, col="blue")
}

}
